All of lore.kernel.org
 help / color / mirror / Atom feed
From: Len Brown <len.brown@intel.com>
To: linux-acpi@vger.kernel.org
Cc: Robert Moore <robert.moore@intel.com>,
	Alexey Starikovskiy <alexey.y.starikovskiy@intel.com>,
	Len Brown <len.brown@intel.com>
Subject: [PATCH 51/65] ACPICA: Fix race condition with AcpiWalkNamespace.
Date: Fri, 24 Nov 2006 02:19:01 -0500	[thread overview]
Message-ID: <11643528124177-git-send-email-len.brown@intel.com> (raw)
Message-ID: <de2576c39bf22ea81e46280dee9a7543176852b1.1164352294.git.len.brown@intel.com> (raw)
In-Reply-To: <11643528111088-git-send-email-len.brown@intel.com>
In-Reply-To: <410c2f0190f74c35505beda6ff3f2da7819f8bac.1164352285.git.len.brown@intel.com>

From: Robert Moore <robert.moore@intel.com>

Fixed a problem with a possible race condition between threads executing
AcpiWalkNamespace and the AML interpreter. This condition was removed by
modifying AcpiWalkNamespace to (by default) ignore all temporary
namespace entries created during any concurrent control method execution

Signed-off-by: Alexey Starikovskiy <alexey.y.starikovskiy@intel.com>
Signed-off-by: Len Brown <len.brown@intel.com>
---
 drivers/acpi/dispatcher/dswload.c |    4 ++--
 drivers/acpi/namespace/nsdump.c   |    9 ++++++++-
 drivers/acpi/namespace/nssearch.c |    4 ++++
 drivers/acpi/namespace/nswalk.c   |   13 ++++++++-----
 include/acpi/aclocal.h            |    2 +-
 include/acpi/acnamesp.h           |   10 +++++++---
 6 files changed, 30 insertions(+), 12 deletions(-)

diff --git a/drivers/acpi/namespace/nsdump.c b/drivers/acpi/namespace/nsdump.c
index da88834..ec5ce59 100644
--- a/drivers/acpi/namespace/nsdump.c
+++ b/drivers/acpi/namespace/nsdump.c
@@ -226,6 +226,12 @@ acpi_ns_dump_one_object(acpi_handle obj_
 	obj_desc = acpi_ns_get_attached_object(this_node);
 	acpi_dbg_level = dbg_level;
 
+	/* Temp nodes are those nodes created by a control method */
+
+	if (this_node->flags & ANOBJ_TEMPORARY) {
+		acpi_os_printf("(T) ");
+	}
+
 	switch (info->display_type & ACPI_DISPLAY_MASK) {
 	case ACPI_DISPLAY_SUMMARY:
 
@@ -623,7 +629,8 @@ acpi_ns_dump_objects(acpi_object_type ty
 	info.display_type = display_type;
 
 	(void)acpi_ns_walk_namespace(type, start_handle, max_depth,
-				     ACPI_NS_WALK_NO_UNLOCK,
+				     ACPI_NS_WALK_NO_UNLOCK |
+				     ACPI_NS_WALK_TEMP_NODES,
 				     acpi_ns_dump_one_object, (void *)&info,
 				     NULL);
 }
diff --git a/drivers/acpi/namespace/nssearch.c b/drivers/acpi/namespace/nssearch.c
index 566f0a4..d261c9b 100644
--- a/drivers/acpi/namespace/nssearch.c
+++ b/drivers/acpi/namespace/nssearch.c
@@ -402,6 +402,10 @@ acpi_ns_search_and_enter(u32 target_name
 	}
 #endif
 
+	if (flags & ACPI_NS_TEMPORARY) {
+		new_node->flags |= ANOBJ_TEMPORARY;
+	}
+
 	/* Install the new object into the parent's list of children */
 
 	acpi_ns_install_node(walk_state, node, new_node, type);
diff --git a/include/acpi/aclocal.h b/include/acpi/aclocal.h
index 553763d..287da6f 100644
--- a/include/acpi/aclocal.h
+++ b/include/acpi/aclocal.h
@@ -204,7 +204,7 @@ struct acpi_namespace_node {
 /* Namespace Node flags */
 
 #define ANOBJ_END_OF_PEER_LIST          0x01	/* End-of-list, Peer field points to parent */
-#define ANOBJ_RESERVED                  0x02	/* Available for future use */
+#define ANOBJ_TEMPORARY                 0x02	/* Node is create by a method and is temporary */
 #define ANOBJ_METHOD_ARG                0x04	/* Node is a method argument */
 #define ANOBJ_METHOD_LOCAL              0x08	/* Node is a method local */
 #define ANOBJ_SUBTREE_HAS_INI           0x10	/* Used to optimize device initialization */
diff --git a/include/acpi/acnamesp.h b/include/acpi/acnamesp.h
index b3b9f0e..19a6129 100644
--- a/include/acpi/acnamesp.h
+++ b/include/acpi/acnamesp.h
@@ -65,9 +65,13 @@
 #define ACPI_NS_ERROR_IF_FOUND      0x08
 #define ACPI_NS_PREFIX_IS_SCOPE     0x10
 #define ACPI_NS_EXTERNAL            0x20
+#define ACPI_NS_TEMPORARY           0x40
 
-#define ACPI_NS_WALK_UNLOCK         TRUE
-#define ACPI_NS_WALK_NO_UNLOCK      FALSE
+/* Flags for acpi_ns_walk_namespace */
+
+#define ACPI_NS_WALK_NO_UNLOCK      0
+#define ACPI_NS_WALK_UNLOCK         0x01
+#define ACPI_NS_WALK_TEMP_NODES     0x02
 
 /*
  * nsinit - Namespace initialization
@@ -92,7 +96,7 @@ acpi_status
 acpi_ns_walk_namespace(acpi_object_type type,
 		       acpi_handle start_object,
 		       u32 max_depth,
-		       u8 unlock_before_callback,
+		       u32 flags,
 		       acpi_walk_callback user_function,
 		       void *context, void **return_value);
 
diff --git a/drivers/acpi/dispatcher/dswload.c b/drivers/acpi/dispatcher/dswload.c
index 565d455..4ed0868 100644
--- a/drivers/acpi/dispatcher/dswload.c
+++ b/drivers/acpi/dispatcher/dswload.c
@@ -756,9 +756,9 @@ acpi_ds_load2_begin_op(struct acpi_walk_
 		flags = ACPI_NS_NO_UPSEARCH;
 		if (walk_state->pass_number == 3) {
 
 			/* Execution mode, node cannot already exist */
 
-			flags |= ACPI_NS_ERROR_IF_FOUND;
+			flags |= (ACPI_NS_ERROR_IF_FOUND | ACPI_NS_TEMPORARY);
 		}
 
 		/* Add new entry or lookup existing entry */
@@ -756,9 +756,9 @@ acpi_ds_load2_begin_op(struct acpi_walk_
 		if (walk_state->pass_number == 3) {
 
-			/* Execution mode, node cannot already exist */
+			/* Execution mode, node cannot already exist, node is temporary */
 
diff --git a/drivers/acpi/namespace/nswalk.c b/drivers/acpi/namespace/nswalk.c
index c8f6bef..a138fcb 100644
--- a/drivers/acpi/namespace/nswalk.c
+++ b/drivers/acpi/namespace/nswalk.c
@@ -126,7 +126,7 @@ struct acpi_namespace_node *acpi_ns_get_
  * PARAMETERS:  Type                - acpi_object_type to search for
  *              start_node          - Handle in namespace where search begins
  *              max_depth           - Depth to which search is to reach
  *              unlock_before_callback- Whether to unlock the NS before invoking
  *                                    the callback routine
  *              user_function       - Called when an object of "Type" is found
  *              Context             - Passed to user function
@@ -153,7 +153,7 @@ acpi_status
 acpi_ns_walk_namespace(acpi_object_type type,
 		       acpi_handle start_node,
 		       u32 max_depth,
-		       u8 unlock_before_callback,
+		       u32 flags,
 		       acpi_walk_callback user_function,
 		       void *context, void **return_value)
 {
@@ -201,12 +201,15 @@ acpi_ns_walk_namespace(acpi_object_type
 				child_type = child_node->type;
 			}
 
-			if (child_type == type) {
+			if ((child_type == type) &&
+			    (!(child_node->flags & ANOBJ_TEMPORARY) ||
+			     (child_node->flags & ANOBJ_TEMPORARY)
+			     && (flags & ACPI_NS_WALK_TEMP_NODES))) {
 				/*
 				 * Found a matching node, invoke the user
 				 * callback function
 				 */
-				if (unlock_before_callback) {
+				if (flags & ACPI_NS_WALK_UNLOCK) {
 					mutex_status =
 					    acpi_ut_release_mutex
 					    (ACPI_MTX_NAMESPACE);
@@ -219,7 +222,7 @@ acpi_ns_walk_namespace(acpi_object_type
 				status = user_function(child_node, level,
 						       context, return_value);
 
-				if (unlock_before_callback) {
+				if (flags & ACPI_NS_WALK_UNLOCK) {
 					mutex_status =
 					    acpi_ut_acquire_mutex
 					    (ACPI_MTX_NAMESPACE);
@@ -126,7 +126,7 @@ struct acpi_namespace_node *acpi_ns_get_
  *              start_node          - Handle in namespace where search begins
  *              max_depth           - Depth to which search is to reach
- *              unlock_before_callback- Whether to unlock the NS before invoking
+ *              Flags               - Whether to unlock the NS before invoking
  *                                    the callback routine
  *              user_function       - Called when an object of "Type" is found
-- 
1.4.4.1

  parent reply	other threads:[~2006-11-24  7:16 UTC|newest]

Thread overview: 67+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2006-11-24  7:18 contents of the acpica branch Len Brown
2006-11-24  7:18 ` [PATCH 1/65] ACPICA: Update function header Len Brown
2006-11-24  7:18   ` Len Brown
2006-11-24  7:18   ` [PATCH 2/65] ACPICA: Handle mis-matched package length Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 3/65] ACPICA: Handle case NumElements > Package length Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 4/65] ACPICA: Delete recursive feature of ACPI Global Lock Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 5/65] ACPICA: Release global lock from interrupt handler Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 6/65] ACPICA: Cast acpi_thread_id to UINT32 for debug output only Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 7/65] ACPICA: fix for object premature deletion Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 8/65] ACPICA: Temporary fix for BankValue parameter Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 9/65] ACPICA: Update version to 20060721 Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 10/65] ACPICA: Update debug output Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 12/65] ACPICA: misc fixes for new Table Manager: Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 13/65] ACPICA: Update comments for individual table fields Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 14/65] ACPICA: Fix for FADT conversion in 64-bit mode Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 15/65] ACPICA: Lint changes Len Brown
2006-11-24  7:18     ` Len Brown
     [not found]       ` <1164352776767-git-send-email-len.brown@intel.com>
2006-11-24  7:18   ` [PATCH 17/65] ACPICA: Add support for DMAR table Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 18/65] ACPICA: Add acpi_gpe_count global to track the number of GPE events Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 19/65] ACPICA: Disable all wake GPEs after first one recieved Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 20/65] ACPICA: Fix unalignment in acpi_ut_repair_name Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 21/65] ACPICA: Store GPE number instead of bitmask Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 22/65] ACPICA: Split acpi_format_exception into two parts Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 23/65] ACPICA: Update version to 20060831 Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 24/65] ACPICA: Cleanup of FADT verification function Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 25/65] ACPICA: Create tbfadt.c to hold all FADT-related functions Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 26/65] ACPICA: Re-implement interpreters' "serialized mode" Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 27/65] ACPICA: Delete stale FADT functions outside tbfadt.c Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 28/65] ACPICA: Update comments in tbfadt.c Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 29/65] ACPICA: add ASF comment Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 30/65] ACPICA: re-factor table init routines for benefit of iASL Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 31/65] ACPICA: Allow type ANY to be the target of the Scope operator Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 32/65] ACPICA: IsResourceTemplate now returns ACPI_STATUS Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 33/65] ACPICA: Add declarations for ASF! sub-tables Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 34/65] ACPICA: FADT verification is now table driven Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 35/65] ACPICA: Report error if method creates 2 objects with the same name Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 36/65] ACPICA: New common routine for creating and verifying a local FADT Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 37/65] ACPICA: Fix memory leak in table load error path Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 38/65] ACPICA: Fix trace output name and whitespace Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 39/65] ACPICA: Update version to 20060912 Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 40/65] ACPICA: Add full table name to disassembler output Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 41/65] ACPICA: Fix for Global Lock semaphore Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 42/65] ACPICA: Remove obsolete Flags parameter Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 43/65] ACPICA: Use faster ByIndex interface to get FACS Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 44/65] ACPICA: On AML mutex force-release, set depth to zero (was 1) Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 45/65] ACPICA: Update interpreter error paths to always report the error Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 46/65] ACPICA: Fix for possible memory leak and fault Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 47/65] ACPICA: Add new subsystem state bit that is set after SubsystemInitialize is called Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 48/65] ACPICA: Update version to 20060927 Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:18   ` [PATCH 49/65] ACPICA: Restructured module into multiple functions Len Brown
2006-11-24  7:18     ` Len Brown
2006-11-24  7:19   ` [PATCH 50/65] ACPICA: Eliminate control method 2-pass parse/execute Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` Len Brown [this message]
2006-11-24  7:19     ` [PATCH 51/65] ACPICA: Fix race condition with AcpiWalkNamespace Len Brown
2006-11-24  7:19   ` [PATCH 52/65] ACPICA: _CID support for PCI Root Bridge detection Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 53/65] ACPICA: Use manifest constants for parse pass number Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 54/65] ACPICA: Update comments Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 55/65] ACPICA: Abort downward walk on temporary node detection Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 56/65] ACPICA: Fixes for parameter validation Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 57/65] ACPICA: Update version to 20061011 Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 58/65] ACPICA: Update debug output routines for data structure changes Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 59/65] ACPICA: Miscellaneous table manager updates and optimizations Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 60/65] ACPICA: Fixes for load() operator Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 61/65] ACPICA: Ensure that all structures in acobject.h are aligned, via #pragma Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 62/65] ACPICA: Add ACPI_MAX macro Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 63/65] ACPICA: Fail AcpiEnable if ACPI tables not loaded Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 64/65] ACPICA: Add include of actables.h Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24  7:19   ` [PATCH 65/65] ACPICA: version update Len Brown
2006-11-24  7:19     ` Len Brown
2006-11-24 20:32 ` contents of the acpica branch Rafael J. Wysocki
2006-11-24 22:25   ` Rafael J. Wysocki
2006-11-27  2:12     ` Len Brown

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=11643528124177-git-send-email-len.brown@intel.com \
    --to=len.brown@intel.com \
    --cc=alexey.y.starikovskiy@intel.com \
    --cc=lenb@kernel.org \
    --cc=linux-acpi@vger.kernel.org \
    --cc=robert.moore@intel.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.